import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter_firebase_basic_tools/logic/multiplatform_webview/multiplatform_webview.dart';
import 'package:flutter_firebase_basic_tools/widgets/class_quiz.dart';
import 'package:flutter_firebase_basic_tools/widgets/language_picker.dart';
import 'package:flutter_firebase_basic_tools/widgets/topbar.dart';
import 'package:flutter_firebase_basic_tools/logic/translate.dart';
import 'package:flutter_firebase_basic_tools/models/class_unit_arguments.dart';
import 'package:flutter_firebase_basic_tools/logic/custom_colors.dart';
import 'package:auto_size_text/auto_size_text.dart';
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/material.dart';
import 'package:universal_platform/universal_platform.dart';

class ClassSession extends StatefulWidget {
  const ClassSession({
    @required this.courseID,
    @required this.courseName,
    @required this.unitID,
    @required this.itemsTextColor,
    @required this.languagePicker,
  });

  final String courseID;
  final String courseName;
  final String unitID;
  final Color itemsTextColor;
  final bool languagePicker;

  @override
  _ClassSessionState createState() => _ClassSessionState();
}

class _ClassSessionState extends State<ClassSession> {
  String currentLanguage = "en";
  TranslationStream translationStream = TranslationStream();
  List<String> textList = [
    "text",
    "text",
    "text",
    "text",
    "text",
  ];

  String videoUrl = "";
  ClassUnitArguments args;
  bool alreadyHaveTexts = false;
  bool fullscreenMode = false;
  String classSessionHtml = "class_session";
  bool lastUnit = false;

  updateTextList(int index, String newText) {
    textList[index] = newText;
    setState(() {});
  }

  String generateVideoUrlForCurrentPlatform(String originalUrl) {
    if (UniversalPlatform.isWeb) {
      return originalUrl;
    } else {
      Uri uri = Uri.parse(originalUrl);
      String newUrl = "https://abeinstitute.com";
      //String newUrl = "https://abeinstitute.com/#/video/${uri.pathSegments.last}";
      //return newUrl;
      return newUrl;
    }
  }

  getClassTexts(String unitID) async {
    await FirebaseFirestore.instance
        .collection('units')
        .doc(unitID)
        .get()
        .then((DocumentSnapshot ds) {
      textList = [
        ds.data()["title"],
        ds.data()["subtitle"],
        ds.data()["description"],
        ds.data()["recomendation"],
        "Class session",
      ];

      videoUrl = generateVideoUrlForCurrentPlatform(
          ds.data()["video_urls"][currentLanguage]);

      lastUnit = ds.data()['last_unit'];

      setState(() {});

      translationStream.init(textList, updateTextList);
      translationStream.translate();

      return null;
    });
  }

  CustomColors customColors = CustomColors();

  languagePickerCallback(String newCurrentLanguage) async {
    currentLanguage = newCurrentLanguage;
    translationStream.translate();
    setState(() {});

    await FirebaseFirestore.instance
        .collection('units')
        .doc(widget.unitID)
        .get()
        .then((DocumentSnapshot ds) {
      videoUrl = generateVideoUrlForCurrentPlatform(
          ds.data()["video_urls"][currentLanguage]);

      setState(() {});

      return null;
    });
  }

  List<Widget> widgetsAction(bool isPortrait) {
    return [
      Container(
        width: isPortrait ? 100 : 150,
        child: LanguagePicker(
          currentLanguage: currentLanguage,
          parentWidget: this,
          function: languagePickerCallback,
          itemsTextColor: CustomColors().oceanBlue,
        ),
      ),
      Container(
        margin: EdgeInsets.only(
            left: MediaQuery.of(context).size.width / 100,
            right: MediaQuery.of(context).size.width / 100),
        child: FlatButton(
          onPressed: () {
            fullscreenMode = !fullscreenMode;
            setState(() {});
          },
          child: Row(
            children: <Widget>[
              Text(
                "Fullscreen",
                style: TextStyle(color: Colors.white),
              ),
              Icon(
                fullscreenMode ? Icons.fullscreen_exit : Icons.fullscreen,
                color: Colors.white,
              ),
            ],
          ),
        ),
      ),
    ];
  }

  @override
  void initState() {
    super.initState();
    getClassTexts(widget.unitID);
  }

  @override
  Widget build(BuildContext context) {
    double appBarSize = 65;

    bool isPortrait =
        MediaQuery.of(context).orientation == Orientation.portrait;

    return Scaffold(
      appBar: PreferredSize(
        preferredSize: Size.fromHeight(appBarSize),
        child: Topbar(
          backgroundColor: customColors.darkAqua.withOpacity(0.7),
          size: appBarSize,
          hasBackButton: true,
          actions: widgetsAction(isPortrait),
          customLeading: IconButton(
            icon: Icon(Icons.arrow_back, color: Colors.white),
            onPressed: () => Navigator.of(context).pop(),
          ),
          logoImage: "assets/images/abei_logo.png",
        ),
      ),
      body: Stack(
        children: <Widget>[
          fullscreenMode
              ? Container()
              : Center(
                  child: Container(
                    child: Column(
                      children: <Widget>[
                        Spacer(flex: 1),
                        Expanded(
                          flex: 1,
                          child: Container(
                            width: isPortrait ? 300 : 500,
                            child: AutoSizeText(
                              textList[0],
                              textAlign: TextAlign.left,
                              style: TextStyle(
                                color: Colors.black,
                                fontSize: 26,
                                fontWeight: FontWeight.bold,
                              ),
                              minFontSize: 22,
                              maxLines: 1,
                              overflow: TextOverflow.clip,
                            ),
                          ),
                        ),
                        Expanded(
                          flex: 1,
                          child: Container(
                            width: isPortrait ? 300 : 500,
                            child: AutoSizeText(
                              textList[1],
                              textAlign: TextAlign.left,
                              style: TextStyle(
                                color: Colors.black,
                                fontSize: 22,
                                fontWeight: FontWeight.bold,
                              ),
                              minFontSize: 18,
                              maxLines: 1,
                              overflow: TextOverflow.clip,
                            ),
                          ),
                        ),
                        Expanded(
                          flex: 2,
                          child: Container(
                            width: isPortrait ? 300 : 500,
                            child: AutoSizeText(
                              textList[2],
                              textAlign: TextAlign.left,
                              style: TextStyle(
                                color: Colors.black,
                                fontSize: 16,
                                fontWeight: FontWeight.bold,
                              ),
                              minFontSize: 12,
                              maxLines: 4,
                              overflow: TextOverflow.clip,
                            ),
                          ),
                        ),
                        Expanded(
                          flex: 5,
                          child: Container(
                            child: MultiplatformWebview(
                              src: videoUrl,
                            ),
                          ),
                        ),
                        Expanded(
                          flex: 2,
                          child: Container(
                            width: isPortrait ? 300 : 500,
                            child: AutoSizeText(
                              textList[3],
                              textAlign: TextAlign.left,
                              style: TextStyle(
                                color: Colors.black,
                                fontSize: 16,
                                fontWeight: FontWeight.bold,
                              ),
                              minFontSize: 12,
                              maxLines: 3,
                              overflow: TextOverflow.clip,
                            ),
                          ),
                        ),
                        RaisedButton(
                          onPressed: () {
                            Navigator.push(
                              context,
                              MaterialPageRoute(
                                builder: (context) => ClassQuiz(
                                  courseID: widget.courseID,
                                  courseName: widget.courseName,
                                  unitID: widget.unitID,
                                  lastUnit: lastUnit,
                                  itemsTextColor: widget.itemsTextColor,
                                  languagePicker: widget.languagePicker,
                                ),
                              ),
                            );
                          },
                          child: Text('Start Quiz'),
                        ),
                        Spacer(flex: 1),
                      ],
                    ),
                  ),
                ),
          fullscreenMode
              ? Container(
                  child: MultiplatformWebview(
                    src: videoUrl,
                  ),
                )
              : Container(),
        ],
      ),
    );
  }
}
